package it.arquilliantest.jpa.controller;

import it.arquilliantest.jpa.entity.Action;
import it.arquilliantest.jpa.exceptions.NonexistentEntityException;
import it.arquilliantest.jpa.exceptions.PreexistingEntityException;
import it.arquilliantest.jpa.exceptions.RollbackFailureException;
import java.io.Serializable;
import java.util.List;
import javax.ejb.Stateless;
import javax.persistence.EntityManager;
import javax.persistence.PersistenceContext;
import javax.persistence.Query;

@Stateless
public class ActionJpaController implements Serializable {

    @PersistenceContext
    private EntityManager em;

    public void create(Action action) throws PreexistingEntityException, RollbackFailureException, Exception {
        em.persist(action);
    }

    public void edit(Action action) throws NonexistentEntityException, RollbackFailureException, Exception {
        em.merge(action);
    }

    public void destroy(Action action) throws NonexistentEntityException, RollbackFailureException, Exception {
            em.remove(action);
    }
    
    public List<Action> findActionEntities() {
        return findActionEntities(true, -1, -1);
    }

    public List<Action> findActionEntities(int maxResults, int firstResult) {
        return findActionEntities(false, maxResults, firstResult);
    }

    private List<Action> findActionEntities(boolean all, int maxResults, int firstResult) {
        Query q = em.createNamedQuery("Action.findAll");
            if (!all) {
                q.setMaxResults(maxResults);
                q.setFirstResult(firstResult);
            }
            return q.getResultList();        
    }

    public Action findAction(Integer id) {
        Query q = em.createNamedQuery("Action.findById");
        q.setParameter("id", id);
        return (Action)q.getSingleResult();        
    }
    
}
