package it.arquilliantest.jpa;

import it.arquilliantest.jpa.controller.ActionJpaController;
import it.arquilliantest.jpa.entity.Action;
import it.arquilliantest.jpa.exceptions.IllegalOrphanException;
import it.arquilliantest.jpa.exceptions.NonexistentEntityException;
import it.arquilliantest.jpa.exceptions.PreexistingEntityException;
import it.arquilliantest.jpa.exceptions.RollbackFailureException;
import it.arquilliantest.jpa.util.ManageExceptionStackTrace;
import java.io.File;
import java.util.MissingResourceException;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.container.test.api.TargetsContainer;
import org.jboss.shrinkwrap.api.GenericArchive;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.*;
import org.jboss.shrinkwrap.api.exporter.ZipExporter;
import org.jboss.shrinkwrap.api.spec.WebArchive;
import org.jboss.shrinkwrap.resolver.api.DependencyResolvers;
import org.jboss.shrinkwrap.resolver.api.maven.MavenDependencyResolver;

/**
 * Set up ShrinkWrap test archive
 * 
 * @author Marco!Cella
 */
public abstract class ArchiveFactory {
    
    private static String className = ArchiveFactory.class.getName();
    
    @Deployment(managed = true)
    @TargetsContainer(value="glassfish-managed")
    public static WebArchive createTestArchive() {       
        WebArchive archive = ShrinkWrap.create(WebArchive.class, "ArquillianTestArchive.war")
                .addClass(ActionJpaController.class)
                .addClass(Action.class)
                .addClasses(IllegalOrphanException.class, NonexistentEntityException.class, PreexistingEntityException.class, RollbackFailureException.class, ArchiveFactory.class, ManageExceptionStackTrace.class)
                .addAsLibraries(DependencyResolvers.use(MavenDependencyResolver.class)
                .artifact("org.hibernate:hibernate-entitymanager:3.5.6-Final")
                .artifact("org.hibernate.javax.persistence:hibernate-jpa-2.0-api:jar:1.0.0.Final")
                .artifact("org.hibernate:hibernate-core:3.5.6-Final")
                .artifact("org.slf4j:slf4j-api:1.6.3")
                .artifact("org.slf4j:slf4j-log4j12:1.6.3")
                .artifact("org.springframework.security:spring-security-core:3.1.0.RELEASE")
                .artifact("org.springframework.security:spring-security-web:3.1.0.RELEASE")
                .artifact("org.springframework.security:spring-security-config:3.1.0.RELEASE")
                .artifact("org.springframework.security:spring-security-ldap:3.1.0.RELEASE")
                .artifact("org.springframework:spring-aop:3.1.1.RELEASE")
                .artifact("org.springframework:spring-asm:3.1.1.RELEASE")
                .artifact("org.springframework:spring-beans:3.1.1.RELEASE")
                .artifact("org.springframework:spring-context:3.1.1.RELEASE")
                .artifact("org.springframework:spring-core:3.1.1.RELEASE")
                .artifact("org.springframework:spring-expression:3.1.1.RELEASE")
                .artifact("org.springframework:spring-jdbc:3.1.1.RELEASE")
                .artifact("org.springframework:spring-orm:3.1.1.RELEASE")
                .artifact("org.springframework:spring-oxm:3.1.1.RELEASE")
                .artifact("org.springframework:spring-tx:3.1.1.RELEASE")
                .artifact("org.springframework:spring-web:3.1.1.RELEASE")
                .artifact("org.springframework.ldap:spring-ldap-core:jar:1.3.1.RELEASE")
                .artifact("org.json:json:20090211")
                .resolveAs(GenericArchive.class))
                .addAsWebInfResource(EmptyAsset.INSTANCE, "beans.xml");
        
        // AS configuration files
        archive.addAsWebInfResource("glassfish-resources.xml", "glassfish-resources.xml");
            
        // DBMS configuration files
        String dbtype = "";
        try {
            ResourceBundle resource =  ResourceBundle.getBundle("test");
            if(resource == null) {
                Logger.getLogger(className).log(Level.SEVERE, "\nArquillian archive class {0} cannot locate 'test.property' file", className);
            } else {
                dbtype = resource.getString("dbtype");
                Logger.getLogger(className).log(Level.INFO, "\nArquillian archive class {0} Database Type: {1}", new Object[] {className, dbtype});
            }
        } catch (MissingResourceException e) {
            Logger.getLogger(className).log(Level.SEVERE, "\nArquillian archive class {0} exception: cannot locate 'test.property' file", className);
        } catch (Exception e) {
            e.printStackTrace();
        }
        if(dbtype == null || (dbtype != null && dbtype.equals("")) || (dbtype != null && dbtype.equals("mysql"))){
            archive.addAsResource("persistence-mysql.xml", "META-INF/persistence.xml");
        } else {
            return null;
        }

        archive.addAsResource("log4j.properties", "log4j.properties");
        
        archive.as(ZipExporter.class).exportTo(
            new File("arquillianTestArchive.war"), true);
        //System.err.println(archive.toString(true));
        return archive;
    }

}