package it.arquilliantest.jpa.controller;

import it.arquilliantest.jpa.ArchiveFactory;
import it.arquilliantest.jpa.entity.Action;
import it.arquilliantest.jpa.exceptions.PreexistingEntityException;
import it.arquilliantest.jpa.exceptions.RollbackFailureException;
import it.arquilliantest.jpa.util.ManageExceptionStackTrace;
import javax.ejb.EJB;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.arquillian.persistence.*;
import static org.junit.Assert.*;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.slf4j.LoggerFactory;
import org.slf4j.Logger;

/**
 * Test ActionJpaController EJB
 *
 * @author Marco!Cella
 */
@RunWith(Arquillian.class)
public class ActionJpaControllerTest extends ArchiveFactory {
    @EJB
    private ActionJpaController actionJpaController;
    
    private static String className = ActionJpaControllerTest.class.getName();
    
    final Logger logger = LoggerFactory.getLogger(ActionJpaControllerTest.class);
    
    // How to handle "Caught exception in beforeCompletion()" exception ?
    // Database table is not cleaned after test execution
    @Test
    @UsingDataSet("jpa/controller/ActionJpaControllerTest/ShouldNotEditAction.xml")
    @ShouldMatchDataSet(value = "jpa/controller/ActionJpaControllerTest/ShouldNotEditActionAssert.xml")
    @Cleanup(phase = TestExecutionPhase.AFTER, strategy = CleanupStrategy.USED_ROWS_ONLY)
    public void shouldNotEditActionOnNullNameField() throws Exception {
        logger.info("\nExecute test: {} [#] shouldNotEditActionOnNullNameField", className);
        // given
        Action givenAction = actionJpaController.findAction( Integer.valueOf(3) );
        assertNotNull( "Action entity must not be null:\n", givenAction );
        Class expectedExceptionClass = javax.persistence.PersistenceException.class;
        boolean testStatus = false;
        String actualException = "No exception";
        try {
            // when
            givenAction.setName( null );
            actionJpaController.edit( givenAction );
            // then
        } catch(Throwable ex){
            testStatus = ManageExceptionStackTrace.existInnerException(ex, expectedExceptionClass);
            if(!testStatus) { actualException = ManageExceptionStackTrace.getInnerException(ex); }
        }
        assertTrue( "Should have throws javax.persistence.PersistenceException: org.hibernate.exception.ConstraintViolationException: Could not execute JDBC batch update.\n" + actualException + " was thown instead.", testStatus );
    }
    
}
